﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;


public class OpenMediationSdkDemo : MonoBehaviour
{
    //publisher app key
    private string appKey;
    public string appKeyIos;
    public string appKeyAndroid;

    //publisher banner placement Id;
    private string bannerPlacementId;
    public string bannerPidIos;
    public string bannerPidAndroid;

    private string splashPlacementId;
    public string splashPidIos;
    public string splashPidAndroid;


    Button btn_splash, btn_rewardedVideo, btn_interstitial, btn_banner, btn_testSuite;

    Button btn_gdprFalse, btn_gdprTrue;

    Button btn_crossPromotion, btn_hideCrossPromotion;

    Text message;

    // Start is called before the first frame update
    void Start()
    {
       #if(UNITY_IPHONE || UNITY_IOS) 
            appKey = appKeyIos;
            bannerPlacementId = bannerPidIos;
            splashPlacementId = splashPidIos;
        #elif UNITY_ANDROID
            appKey = appKeyAndroid;
            bannerPlacementId = bannerPidAndroid;
            splashPlacementId = splashPidAndroid;
        #endif

        Debug.LogError("start");
        btn_splash = GameObject.Find("Splash").GetComponent<Button>();
        btn_rewardedVideo = GameObject.Find("RewardedVideo").GetComponent<Button>();
        btn_interstitial = GameObject.Find("Interstitial").GetComponent<Button>();
        btn_banner = GameObject.Find("Banner").GetComponent<Button>();
        message = GameObject.Find("Message").GetComponent<Text>();

        btn_gdprTrue = GameObject.Find("GDPRTrue").GetComponent<Button>();
        btn_gdprFalse = GameObject.Find("GDPRFalse").GetComponent<Button>();

        btn_testSuite = GameObject.Find("TestSuite").GetComponent<Button>();

        btn_crossPromotion = GameObject.Find("ShowCrossPromotion").GetComponent<Button>();
        btn_hideCrossPromotion = GameObject.Find("HideCrossPromotion").GetComponent<Button>();

        btn_rewardedVideo.interactable = false;
        btn_interstitial.interactable = false;
        btn_banner.interactable = false;

        btn_crossPromotion.interactable = false;
        btn_hideCrossPromotion.interactable = false;


        AddEvent();

        btn_splash.onClick.AddListener(LoadAndShowSplash);

        btn_rewardedVideo.onClick.AddListener(ShowRewardedVideo);

        btn_interstitial.onClick.AddListener(ShowInterstitial);

        btn_banner.onClick.AddListener(LoadAndShowBanner);

        btn_gdprTrue.onClick.AddListener(setGdprTrue);
        btn_gdprFalse.onClick.AddListener(setGdprFalse);

        btn_testSuite.onClick.AddListener(ShowTestSute);

        btn_crossPromotion.onClick.AddListener(ShowCrossPromotion);
        btn_hideCrossPromotion.onClick.AddListener(HideCrossPromotion);
        InitSdk();
        
    }


    void setGdprTrue() {
        UpdateInfo("setGDPR true");
        Om.Agent.setGDPRConsent(true);
    }


    void setGdprFalse() {
        UpdateInfo("setGDPR false");
        Om.Agent.setGDPRConsent(false);
    }

    void ShowTestSute() {
        OMTestSuite.Show(appKey);
    }


    void AddEvent() {
        //Add Event For Sdk Init
        OmEvents.onSdkInitSuccessEvent += OnInitSuccess;
        OmEvents.onSdkInitFailedEvent += OnInitError;
        //Add Event For RewardedVideo
        OmEvents.onRewardedVideoAvailabilityChangedEvent += RewardedVideoAvailabilityChangedEvent;
        OmEvents.onInterstitialShowFailedEvent += RewardedVideoAdShowFailedEvent;
        OmEvents.onRewardedVideoClickedEvent += RewardedVideoClickedEvent;
        OmEvents.onRewardedVideoClosedEvent += RewardedVideoAdClosedEvent;
        OmEvents.onRewardedVideoStartedEvent += RewardedVideoAdStartedEvent;
        OmEvents.onRewardedVideoEndedEvent += RewardedVideoAdEndedEvent;
        OmEvents.onRewardedVideoShowFailedEvent += RewardedVideoAdShowFailedEvent;
        OmEvents.onRewardedVideoRewardedEvent += RewardedVideoAdRewardedEvent;
        OmEvents.onRewardedVideoShowedEvent += RewardedVideoAdShowEvent;

        //Add Event For Interstitial
        OmEvents.onInterstitialAvailabilityChangedEvent += InterstitialAdAvailabilityChangedEvent;
        OmEvents.onInterstitialShowedEvent += InterstitialAdShowEvent;
        OmEvents.onInterstitialShowFailedEvent += InterstitialAdShowFaiedEvent;
        OmEvents.onInterstitialClickedEvent += InterstitialAdClickedEvent;
        OmEvents.onInterstitialClosedEvent += InterstitialAdClosedEvent;


        //Add Event for Cross Promotion
        OmEvents.onPromotionAdAvailabilityChangedEvent += PromotionAdAvailabilityChangedEvent;
        OmEvents.onPromotionAdClickedEvent += PromotionAdClickedEvent;
        OmEvents.onPromotionAdShowedEvent += PromotionAdShowedEvent;
        OmEvents.onPromotionAdShowFailedEvent += PromotionAdShowFailedEvent;
        OmEvents.onPromotionAdHiddenEvent += PromotionAdHiddenEvent;

        //Add Event For Banner
        OmEvents.onBannerLoadSuccessEvent += BannerLoadSuccessEvent;
        OmEvents.onBannerClickedEvent += BannerClickedEvent;
        OmEvents.onBannerLoadFailedEvent += BannerLoadFailedEvent;


        //Impression Data
        OmEvents.onImpressionDataCallback += ImpressionData;
        OmEvents.onImpressionDataErrorCallback += ImpressionDataError;

        //Splash
        OmEvents.onSplashAdLoadSuccessEvent += SplashAdLoadSuccessEvent;
        OmEvents.onSplashAdLoadFailedEvent += SplashAdLoadFailedEvent;
        OmEvents.onSplashAdShowedEvent += SplashAdShowedEvent;
        OmEvents.onSplashAdClickEvent += SplashAdClickEvent;
        OmEvents.onSplashAdClosedEvent += SplashAdClosedEvent;
        OmEvents.onSplashAdShowFailedEvent += SplashAdShowFailedEvent;

    }

    //init openmeidiation sdk
    void InitSdk() {
        
        UpdateInfo("Start Init Sdk");
        UpdateInfo("appKey:" + appKey);
        Om.Agent.init(appKey);
        //btn_init.interactable = false;
    }

    //Receive message when sdk init success
    void OnInitSuccess() {
        UpdateInfo("OpenMediation Sdk Init Success");
        btn_banner.interactable = true;
        Debug.LogError("isInitOnSuccess " + Om.Agent.isInitialized());
        Om.Agent.debug(true);
        


    }

    //Receive message when sdk init failed
    void OnInitError(string error) { 
        UpdateInfo("OpenMediation Sdk Init Error " + error);
        Debug.LogError("isInitOnError " + Om.Agent.isInitialized());
    }

    //Update Text and print log
    void UpdateInfo(string msg) {
        message.text = msg;
        Debug.LogError(msg);
    }


    /**
     * RewardedVideo Event
     * 
     */

    // bool buttonInteractable = false;

    //在RewardedVideoAvailabilityChangedEvent的回调中更新按钮状态。
    //若不在当前回调中设置，可以记录当前状态作为后续设置的依据。
    void RewardedVideoAvailabilityChangedEvent(bool canShow) {
        //记录状态，用于在需要时设置按钮状态。
        //buttonInteractable = canShow;

        UpdateInfo("RewardedVideo ad can show :" + canShow);

        //在回调中直接设置按钮状态。
        btn_rewardedVideo.interactable = canShow;
    }


    void RewardedVideoAdShowEvent(string scene) {
        UpdateInfo("rewardedVideo ad show scene: " + scene);
    }

    void RewardedVideoAdRewardedEvent(string scene) {
        UpdateInfo("rewardedVideo ad rewarded scene: " + scene);
        //TODO IF this Event Received， you should reward users.

    }

    void RewardedVideoAdEndedEvent(string scene) {
        UpdateInfo("rewardedVideo ad play ended scene: " + scene);
    }

    void RewardedVideoAdClosedEvent(string scene) {
        UpdateInfo("rewardedVideo ad closed scene: " + scene);
        //TODO Here resume your game.
    }

    void RewardedVideoAdStartedEvent(string scene) {
        UpdateInfo("rewardedVideo ad startd event scene : " + scene);
    }

    void RewardedVideoAdShowFailedEvent(string error) {
        UpdateInfo("rewardedVideo ad show failed error : " + error);
        //TODO Here resume your game.
    }

    void RewardedVideoClickedEvent(string scene) {
        UpdateInfo("rewardedVideo ad clicked scene: " + scene);
    }


    /**
     * show rewardedVideo
     */
    //Click button and show rewarded Video ad;
    void ShowRewardedVideo() {
        //Determine if the ad is ready before showing
        if (Om.Agent.isRewardedVideoReady())
        {
            //Show Ad
            //TODO Here pause game and show Ad
            Om.Agent.showRewardedVideo();
        }
        else {
            UpdateInfo("rewardedVideo Ad not ready");
            
        }
    }


    /**
     * interstiital event
     */

    void InterstitialAdAvailabilityChangedEvent(bool canShow) {
        UpdateInfo("Interstitial ad can show :" + canShow);
        btn_interstitial.interactable = canShow;
    }

    void InterstitialAdShowFaiedEvent(string error) {
        UpdateInfo("Interstitial ad show failed error: " + error);
    }

    void InterstitialAdShowEvent(string scene) {
        UpdateInfo("Interstitial ad show scene:" + scene);
    }

    void InterstitialAdClickedEvent(string scene) {
        UpdateInfo("Interstitial ad clicked scene: " + scene);
    }

    void InterstitialAdClosedEvent(string scene) {
        UpdateInfo("Interstitial ad closed scene: " + scene);
    }



    /**
     * Cross Promotion Event
     */

    void PromotionAdAvailabilityChangedEvent(bool canShow)
    {
        UpdateInfo("CrossPromotion ad can show : " + canShow);
        btn_crossPromotion.interactable = true;
    }


    void PromotionAdShowFailedEvent(string error)
    {
        UpdateInfo("PromotionAdShowFailedEvent code: " + error);
    }

    void PromotionAdClickedEvent(string scene)
    {
        UpdateInfo("PromotionAdClickedEvent : " + scene);
    }

    void PromotionAdShowedEvent(string scene)
    {
        UpdateInfo("PromotionAdShowedEvent " + scene);
        btn_crossPromotion.interactable = false;
        btn_hideCrossPromotion.interactable = true;
    }

    void PromotionAdHiddenEvent(string scene)
    {
        UpdateInfo("PromotionAdHiddenEvent: " + scene);
        btn_hideCrossPromotion.interactable = false;
        btn_crossPromotion.interactable = true;

    }


    void ShowCrossPromotion() {
        if (Om.Agent.isPromotionAdReady())
        {
            Om.Agent.showPromotionAd(150, 300, 0.1f, 0.2f, 20);
        }
        else {
            UpdateInfo("CrossPromotion Ad not ready");
        }
       
    }

    void HideCrossPromotion() {
        Om.Agent.hidePromotionAd();
    }
   

    /**
     * show interstitial
     */

    void ShowInterstitial() {
        if (Om.Agent.isInterstitialReady())
        {
            Om.Agent.showInterstitial();
           
        }
        else {
            UpdateInfo("Interstitial ad not ready");
        }
    }

    /**
     * load and show banner
     */

    void BannerLoadSuccessEvent(string placementId) {
        btn_banner.interactable = true;
        UpdateInfo("banner load success " + placementId);
        Om.Agent.displayBanner(bannerPlacementId);
    }

    void BannerLoadFailedEvent(string placementId, string error) {
        btn_banner.interactable = true;
        UpdateInfo("banner load failed " + placementId + " error : " + error);
    }

    void BannerClickedEvent(string placementId) {
        UpdateInfo("banner ad clicked " + placementId);
    }

    void LoadAndShowBanner() {
        btn_banner.interactable = false;
        UpdateInfo("banner loading...");
        Om.Agent.loadBanner(bannerPlacementId, AdSize.BANNER, BannerPostion.BOTTOM);
    }


    /**
     * impresson data event
     */
    void ImpressionData(OmImpressionData impressionData) {
        UpdateInfo("impresssion data : " + impressionData.allData);

    }
    void ImpressionDataError(string error) {
        UpdateInfo("Get impression data error " + error);
    }


    /**
     * Splash
     */

    void SplashAdLoadSuccessEvent(string placementId) {
        UpdateInfo("SplashAdLoadSuccessEvent: " + placementId);
        if (Om.Agent.isSplashAdReady(placementId))
        {
            Om.Agent.showSplashAd(placementId);
        }
    }

    void SplashAdLoadFailedEvent(string placementId) {
        UpdateInfo("SplashAdLoadFailedEvent: " + placementId);
        btn_splash.interactable = true;
    }

    void SplashAdShowedEvent(string placementId) {
        UpdateInfo("SplashAdShowedEvent: " + placementId);
    }

    void SplashAdShowFailedEvent(string placementId) {
        UpdateInfo("SplashAdShowFailedEvent: " + placementId);
        btn_splash.interactable = true;
    }

    void SplashAdClickEvent(string placementId) {
        UpdateInfo("SplashAdClickEvent: " + placementId);
    }

    void SplashAdClosedEvent(string placementId) {
        UpdateInfo("SplashAdClosedEvent: " + placementId);
        btn_splash.interactable = true;
    }

    void LoadAndShowSplash() {
        Om.Agent.loadSplashAd(splashPlacementId);
        btn_splash.interactable = false;
    }

}
